package sihl

  1. Overview
  2. Docs

Module Web.User

exception User_not_found
val find : Rock.Request.t -> Sihl__.Contract_user.t

find req returns the user that is associated with the bearer token. If no user is found, User_not_found is raised.

val find_opt : Rock.Request.t -> Sihl__.Contract_user.t option

find_opt req returns the user that is associated with the bearer token.